#!/usr/bin/env ruby
#
# action.rb
# BioX Database Client
#
# Created by Erik Lagercrantz on 2008-02-05.
# Copyright 2008 Erik Lagercrantz.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#

require 'uri'

require 'service'
require 'command'
require 'provider/common'

module DatabaseClient
  class Action
    ACTION_REGEXP = /^((\w{1,99})[:.\/])?(\w{1,99})(@([[:graph:]]{1,999}))?$/
    ACTION_REGEXP_SERVICE_INDEX = 2
    ACTION_REGEXP_COMMAND_INDEX = 3
    ACTION_REGEXP_LOCATION_INDEX = 5
    
    attr_reader :service, :command, :location, :arguments
    
    def initialize(string, arguments)
      match_results = ACTION_REGEXP.match(string)
      usage_error "Invalid action `#{string}'." unless match_results
      
      service_name = match_results[ACTION_REGEXP_SERVICE_INDEX]      
      command_name = match_results[ACTION_REGEXP_COMMAND_INDEX]
      location_string = match_results[ACTION_REGEXP_LOCATION_INDEX]
      
      if service_name
        @service = Service.filter(service_name).min {|a,b| a.length <=> b.length}
        usage_error "Unknown service `#{service_name}'." unless @service
      else
        @service = Service::Common.default
      end
      
      @command = @service.filter_commands(command_name).min {|a,b| a.length <=> b.length}
      usage_error "Unrecognized command `#{command_name}'." unless @command
      
      if location_string
        begin
          @location = URI.parse(location_string)
        rescue URI::InvalidURIError => e
          usage_error "Location `#{location_string}' is not a valid URI."
        end
      else
        @location = nil
      end
      
      @arguments = arguments.dup
    end
    
    def perform
      begin
        command.invoke(arguments, location)
      rescue Command::HelpFlagIncluded => e
        show_help e.message
      rescue Command::ParameterError => e
        usage_error e.message
      rescue Command::OperationError => e
        execution_error e.message
      end
    end
    
    def service_string
      (service.identifier if service).to_s
    end
    
    def command_string
      (command.identifier if command).to_s
    end
    
    def location_string
      location.to_s
    end
    
    def arguments_string
      (arguments * ' ' if arguments).to_s
    end
    
    def sc_string
      ([service_string, command_string] - [nil, '']) * '.'
    end
    
    def scu_string
      srv = service_string == '' ? 'service' : service_string
      ([srv, command_string] - [nil, '']) * '.'
    end
    
    def scl_string
      ([sc_string, location_string] - [nil, '']) * '@'
    end
    
    def to_s
      ([scl_identifier, arguments_string] - [nil, '']) * ' '
    end
        
  private  
    def show_help(message = nil)
      raise(ShowHelp.new(self, message))
    end

    def usage_error(message = nil)
      raise(UsageError.new(self, message))
    end
    
    def execution_error(message = nil)
      raise(ExecutionError.new(self, message))
    end    
  end
  
  class UsageError < RuntimeError
    attr_accessor :action
    def initialize(action, message)
      @action = action
      super(message)
    end
  end
  
  class ShowHelp < UsageError
  end
    
  class ExecutionError < RuntimeError
    attr_accessor :action
    def initialize(action, message)
      @action = action
      super(message)
    end
  end
end
