#!/usr/bin/env ruby
#
# command.rb
# BioX Database Client
#
# Created by Erik Lagercrantz on 2008-02-05.
# Copyright 2008 Erik Lagercrantz.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#

# Base class for commands that the user can invoke.

require 'param'
require 'spartan'
require 'traits'

require 'ostruct'

module DatabaseClient
  module Command
    class Base
      
      ## Default version of the command
      
      include InheritableTraits
      traits :default_identifier, :default_description, :default_standard_uri, :supported_schemes
      
      def self.default
        self.new(default_identifier, 
                 default_description, 
                 default_standard_uri)
      end
      
      ## Instance methods
      
      attr_reader :identifier, :description, :standard_uri
      
      def initialize(identifier, description, standard_uri = nil)
        @identifier = identifier
        @description = description
        @standard_uri = standard_uri
        @parser = nil
        @spartan = nil
      end
      
      def parser
        if @parser.nil?
          @parser = ParameterParser.new
          configure_parser
        end
        @parser
      end
      
      def spartan
        if @spartan.nil?
          @spartan = Spartan.new
        end
        @spartan
      end
      
      def identified_by?(string)
        self.identifier.index(string) == 0
      end
      
      def invoke(arguments, uri = nil)
        uri ||= self.standard_uri
        if !uri.nil? && !self.class.supported_schemes.include?(uri.scheme)
          parameter_error("Schema '#{uri.scheme}' not supported by command.")
        end
        
        options = parse_args(arguments)
        options.uri = uri
        
        run(options, uri)
      end
      
      def arguments_format
        "[options]"
      end
      
      def arguments_description
        string = ""
        parser.options.each do |opt|
          option = opt[1]
          string << "  #{option.short}  #{option.desc}\n"
        end
        string
      end
      
      def help_text
        "Help for #{self.identifier}.\n"
      end      

    private
      
      def configure_parser
        parser.option :help do
          short '-h'
          long '--help'
          desc "Show this help message"
        end
        parser.option :debug do
          short '-d'
          long '--debug'
          desc "Turn on debugging mode"
        end
      end
      
      def parse_args(arguments)
        begin
          result = parser.parse(arguments)
        rescue Choice::Parser::UnknownOption => e
          parameter_error("Unrecognized option `#{e.message}'.")
        rescue Choice::Parser::ArgumentRequired => e
          parameter_error("Argument required for option `#{e.message}'.")
        rescue Choice::Parser::InvalidArgument => e
          parameter_error("Invalid argument given for `#{e.message}'.")
        else
          help_flag_included if result[:help]
        end
        result
      end
      
      def run(options, uri)
        p options
        p uri
      end
      
      def not_implemented
        raise(OperationError, "Not implemented.")
      end
      def parameter_error(message = nil)
        raise(ParameterError, message)
      end
      def operation_error(message = nil)
        raise(OperationError, message)
      end
      def help_flag_included(message = nil)
        raise(HelpFlagIncluded, message)
      end
      
      
      def nameify(identifier)
        acronyms = /(^EMBL|^EBI|PDB|WGS|ASTD|SBO)/i
        words = identifier.gsub(acronyms, ' \1 ').strip.split(/[ ._-]+/)
        name = words.map{|w| w.capitalize}.join(' ')
        name = name.gsub(acronyms) {|a| a.upcase}
      end
         
    end # Base

    class ParameterError < ArgumentError
    end
    class HelpFlagIncluded < ParameterError
    end
    class OperationError < RuntimeError
    end

  end # Command
end # DatabaseClient
