#!/usr/bin/env ruby
#
# db.rb
# BioX Database Client
#
# Created by Erik Lagercrantz on 2008-01-29.
# Copyright 2008 Erik Lagercrantz.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#

# BioX Database Client main program.

$:.unshift File.dirname(__FILE__)

require 'action'
require 'service'

require 'provider/ebi'
require 'provider/mrs'
require 'provider/ncb'

module DatabaseClient
  class Application
    VERSION = '1.0'
  
    attr_reader :arguments
  
    def initialize(arguments)
      @arguments = arguments
    end
  
    ## Startup

    def run
      # Look at first argument
      action_string = arguments.shift
      if (action_string.nil?)
        show_mini_help
        exit 1
      elsif (["help", "h", "--help", "-h"].include? action_string)
        begin
          action_string = arguments.shift
          action = Action.new(action_string, arguments) if action_string
        rescue UsageError => e
          STDERR.puts(([$0, e.action.sc_string, e.message] - [nil, '']) * ': ')
        end
        show_help(action)
        exit 0
      else
        begin
          action = Action.new(action_string, arguments)
          action.perform
        rescue Interrupt => e
          STDERR.puts " Interrupted."
          raise
        rescue ShowHelp => e
          show_help(e.action)
          exit 0
        rescue UsageError => e
          STDERR.puts(([$0, e.action.sc_string, e.message] - [nil, '']) * ': ')
          show_mini_help(e.action)
          exit 1
        rescue ExecutionError => e
          STDERR.puts(([$0, e.action.scl_string, e.message] - [nil, '']) * ': ')
          exit 2
        else
          exit 0
        end
      end
    end
  
    ## General help messages
    
    def show_mini_help(action = nil)
      service_str = (action.service_string + '.' if action and action.service_string != '').to_s
      command_str = (' ' + action.command_string if action and action.command_string !~ /^(help)?$/).to_s
      STDERR.puts "Type `#{$0} #{service_str}help#{command_str}' for usage."
    end

    def show_help(action = nil)
      if action and action.command
        desc = action.command.description.to_s
        help = action.command.help_text.to_s
        argfmt = action.command.arguments_format.to_s
        argdesc = action.command.arguments_description.to_s
        
        STDERR.print "#{action.sc_string}: #{desc}\n" if desc != ""
        STDERR.print "Usage: #{$0} #{action.scu_string}[@location]"
        STDERR.print " #{argfmt}" if argfmt != ""
        STDERR.print "\n"
        STDERR.print "\n#{help}" if help != ""
        STDERR.print "\nValid options:\n#{argdesc}" if argdesc != ""
      else
        puts "Usage: #{$0} <service>.<command>[@location] [args] [options]"
        puts "       #{$0} help [command]"
        puts ""
        puts "Available services:"
        Service.all.each do |s|
          puts "  #{s.identifier} - #{s.name}"
        end
        puts ""
        puts "Common subcommands:"
        Service::Common.default.all_commands.each do |c|
          puts "  #{c.identifier} - #{c.description}"
        end
        puts ""
        puts "The format of the location string depends on the service."
        puts ""
        puts "Type '#{$0} help <command>' or '#{$0} <service>.help' for details."
      end
    end

  end # App
end # DatabaseModule

app = DatabaseClient::Application.new(ARGV)
app.run
exit 0
