#!/usr/bin/env ruby
#
# common.rb
# BioX Database Client
#
# Created by Erik Lagercrantz on 2008-02-05.
# Copyright 2008 Erik Lagercrantz.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#

# Commands that are common for all database services.

require 'service'
require 'command'

module DatabaseClient
  
  module Service
    class Common < Base
      def self.default
        self.new(nil, nil)
      end
      
      def initialize(identifier, name)
        super(identifier, name)
        self.register_command(Command::Common::Help.default)
        self.register_command(Command::Common::List.default)
        self.register_command(Command::Common::Info.default)
        self.register_command(Command::Common::Query.default)
        self.register_command(Command::Common::BLAST.default)
        self.register_command(Command::Common::Fetch.default)
      end
    end # Common
  end # Service
  
  module Command
    module Common
      class Help < Base
        default_identifier 'help'
        default_description "Show usage information"
        
        def arguments_format
          ""
        end
        def arguments_description
          ""
        end
        
      private
        
        def configure_parser
          super
        end
        
        def parse_args(arguments)
          super(arguments)
        end
        
        def run(options, uri)
          puts "General help"
        end
      end
      
      class List < Base
        default_identifier 'list'
        default_description "List available databases."
      private        
        def configure_parser
          super
          parser.option :format do
            short '-f'
            long '--format=FORMAT'
            desc "The format to use for the output (default table)"
          end
          parser.option :type do
            short '-t'
            long '--type=TYPE'
            desc "The types of databases to include in the list (default all)."
          end
          parser.option :info do
            short '-i'
            long '--info=INFO'
            desc "Information to include in the list (default name)."
          end
        end
        
        def parse_args(arguments)
          super(arguments)
        end
        
        def run(options, uri)
          parameter_error("Service name required.")
        end
        
        def show_database(dbhash, options)
          spartan.context 'database' do |db|
            db.properties dbhash
          end
        end
      end
      
      class Info < Base
        default_identifier 'info'
        default_description "Show information about a database."
        
        def arguments_format
          "<database> [options]"
        end
        
      private
        
        def configure_parser
          super
          parser.option :format do
            short '-f'
            long '--format=FORMAT'
            desc "The format to use for the output (default text)"
          end
        end
        
        def parse_args(arguments)
          options = super(arguments)
          options.database = arguments.shift || parameter_error("Database name required.")
          options
        end
        
        def run(options, uri)
          parameter_error("Service name required.")
        end
        
        def show_database(dbhash, options)
          spartan.properties dbhash
        end
      end
      
      class Query < Base
        default_identifier 'query'
        default_description "Search for database entries using metadata."
        
        def arguments_format
          "<database> <query> [options]"
        end
        
      private
        
        def configure_parser
          super
          parser.option :format do
            short '-f'
            long '--format=FORMAT'
            desc "The format to use for the output (default table)"
            default 'table'
            valid %w[table spartan]
          end
          parser.option :maxcount do
            short '-n'
            long '--max=N'
            desc "Show at most N hits (default 1000)"
            default 1000
            cast Integer
          end
          parser.option :offset do
            short '-o'
            long '--offset=N'
            desc "Skip the N first hits (default 0)"
            default 0
            cast Integer
          end
        end
        
        def parse_args(arguments)
          options = super(arguments)
          options.database = arguments.shift || parameter_error("Database name required.")
          options.query = arguments.shift || parameter_error("Query argument required.")
          options
        end
        
        def run(options, uri)
          parameter_error("Service name required.")
        end
      end
      
      class BLAST < Base
        default_identifier 'blast'
        default_description "Search for database entries using homology."
        
        def arguments_format
          "<database> <program> [options]"
        end
        
      private
        
        def configure_parser
          super
          parser.option :format do
            short '-f'
            long '--format=FORMAT'
            desc "The format to use for the output (default table)"
          end
        end
        
        def parse_args(arguments)
          options = super(arguments)
          options.database = arguments.shift || parameter_error("Database name required.")
          options.program = arguments.shift || parameter_error("Program name required.")
          options
        end
        
        def run(options, uri)
          parameter_error("Service name required.")
        end
      end
      
      class Fetch < Base
        default_identifier 'fetch'
        default_description "Get a database entry by identifier."
        
        def arguments_format
          "<database> <entry1> [entry2 ...]"
        end
        
      private
        
        def configure_parser
          super
        end
        
        def parse_args(arguments)
          options = super(arguments)
          options.database = arguments.shift
          options.identifiers = arguments
          options
        end
        
        def run(options, uri)
          parameter_error("Service name required.")
        end
      end
    end # Common
  end # Command
  
end # DatabaseClient
