#!/usr/bin/env ruby
#
# ebi.rb
# BioX Database Client
#
# Created by Erik Lagercrantz on 2008-02-05.
# Copyright 2008 Erik Lagercrantz.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#

require 'provider/common'

require 'ws/ebi/fetch'
require 'ws/ebi/search'
require 'ws/ebi/wublast'

require 'reporter/ewb'

module DatabaseClient

  module Command
    module EBI
      SEARCH_URI = URI.parse(SOAP::EBI::Search::Driver::DefaultEndpointUrl)
      FETCH_URI = URI.parse(SOAP::EBI::Fetch::Driver::DefaultEndpointUrl)
      WUBLAST_URI = URI.parse(SOAP::EBI::WUBLAST::Driver::DefaultEndpointUrl)
      
      class Help < Common::Help
      private
        def run(options, uri)
          puts "EBI help"
        end
      end # Help

      class List < Common::List
        BLAST_PROTEIN_DATABASES = [
          'uniprot', 'swissprot', 'uniref100', 'uniref90', 'uniref50', 'uniparc', 'ipi', 'pdb', 'sgt', 'intact',
          'imgthlap', 'epop', 'jpop', 'uspop'
        ]
        BLAST_NUCLEOTIDE_DATABASES = [
          'embl', 'em_rel', 'emnew', 'emcds', 'em_rel_env', 'em_rel_est_env', 'em_rel_gss_env', 'em_rel_htg_env',
          'em_rel_pat_env', 'em_rel_std_env', 'em_rel_fun', 'em_rel_est_fun', 'em_rel_gss_fun', 'em_rel_htc_fun',
          'em_rel_htg_fun', 'em_rel_pat_fun', 'em_rel_std_fun', 'em_rel_sts_fun', 'em_rel_tpa_fun', 'em_rel_hum',
          'em_rel_est_hum', 'em_rel_gss_hum', 'em_rel_htc_hum', 'em_rel_htg_hum', 'em_rel_pat_hum', 'em_rel_std_hum',
          'em_rel_sts_hum', 'em_rel_tpa_hum', 'em_rel_inv', 'em_rel_est_inv', 'em_rel_gss_inv', 'em_rel_htc_inv',
          'em_rel_htg_inv', 'em_rel_pat_inv', 'em_rel_std_inv', 'em_rel_sts_inv', 'em_rel_tpa_inv', 'em_rel_mam',
          'em_rel_est_mam', 'em_rel_gss_mam', 'em_rel_htc_mam', 'em_rel_htg_mam', 'em_rel_pat_mam', 'em_rel_std_mam',
          'em_rel_sts_mam', 'em_rel_tpa_mam', 'em_rel_mus', 'em_rel_est_mus', 'em_rel_gss_mus', 'em_rel_htc_mus',
          'em_rel_htg_mus', 'em_rel_pat_mus', 'em_rel_std_mus', 'em_rel_sts_mus', 'em_rel_tpa_mus', 'em_rel_phg',
          'em_rel_gss_phg', 'em_rel_htg_phg', 'em_rel_pat_phg', 'em_rel_std_phg', 'em_rel_tpa_phg', 'em_rel_pln',
          'em_rel_est_pln', 'em_rel_gss_pln', 'em_rel_htc_pln', 'em_rel_htg_pln', 'em_rel_pat_pln', 'em_rel_std_pln',
          'em_rel_sts_pln', 'em_rel_tpa_pln', 'em_rel_pro', 'em_rel_est_pro', 'em_rel_gss_pro', 'em_rel_htc_pro',
          'em_rel_htg_pro', 'em_rel_pat_pro', 'em_rel_std_pro', 'em_rel_sts_pro', 'em_rel_tpa_pro', 'em_rel_rod',
          'em_rel_est_rod', 'em_rel_gss_rod', 'em_rel_htc_rod', 'em_rel_htg_rod', 'em_rel_pat_rod', 'em_rel_std_rod',
          'em_rel_sts_rod', 'em_rel_tpa_rod', 'em_rel_syn', 'em_rel_pat_syn', 'em_rel_std_syn', 'em_rel_tpa_syn',
          'em_rel_tgn', 'em_rel_std_tgn', 'em_rel_unc', 'em_rel_pat_unc', 'em_rel_std_unc', 'em_rel_vrl',
          'em_rel_gss_vrl', 'em_rel_htg_vrl', 'em_rel_pat_vrl', 'em_rel_std_vrl', 'em_rel_tpa_vrl', 'em_rel_vrt',
          'em_rel_est_vrt', 'em_rel_gss_vrt', 'em_rel_htc_vrt', 'em_rel_htg_vrt', 'em_rel_pat_vrt', 'em_rel_std_vrt',
          'em_rel_sts_vrt', 'em_rel_tpa_vrt', 'em_rel_est', 'em_rel_gss', 'em_rel_htc', 'em_rel_htg', 'em_rel_pat',
          'em_rel_std', 'em_rel_sts', 'em_rel_tpa', 'emall', 'evec', 'imgtligm', 'imgthla', 'nem', 'fil', 'brugia',
          'oncv', 'apicom', 'plas', 'plasf', 'toxo', 'crypto', 'eimeria', 'kineto', 'schisto', 'schunq', 'mansoni',
          'jap', 'cercunq', 'japunq', 'entamoeba', 'bgESTnr', 'DB', 'astdi', 'astdg', 'astdp'
        ]
        supported_schemes ['http']
        default_standard_uri SEARCH_URI
      private

        def parse_args(arguments)
          super
        end

        def run(options, uri)
          BLAST_PROTEIN_DATABASES.each do |identifier|
            show_database({
              'id' => identifier,
#              'name' => nameify(identifier),
              'fetch' => false,
              'metadata' => false,
              'homology' => true,
              'moltype' => 'prot'},
              options)
          end
          BLAST_NUCLEOTIDE_DATABASES.each do |identifier|
            show_database({
              'id' => identifier,
#              'name' => nameify(identifier),
              'fetch' => false,
              'metadata' => false,
              'homology' => true,
              'moltype' => 'nuc'},
              options)
          end
          
          # ebeye = SOAP::EBI::Search::Driver.new()
          # ebeye.wiredump_dev = STDERR if options.debug
          # 
          # begin
          #   parameters = SOAP::EBI::Search::ListDomains.new
          #   response = ebeye.listDomains(parameters)
          # rescue REXML::ParseException => e
          #   operation_error("The server returned an invalid response.")
          # rescue Exception => e
          #   operation_error(e.message)            
          # end
          # 
          # response.arrayOfDomainNames.each do |database|
          #   show_database({'id' => database, 'name' => nameify(database)}, options)
          # end
          
          # puts "\nFetch:"
          # puts FETCH_URI
          # dbfetch = SOAP::EBI::Fetch::Driver.new()
          # dbfetch.wiredump_dev = STDERR if options.debug
          # 
          # begin
          #   response = dbfetch.getSupportedDBs
          # rescue Exception => e
          #   operation_error(e.message)
          # end
          # 
          # response.each do |database|
          #   show_database({'id' => database, 'name' => nameify(database)}, options)
          # end
          
        end
      end # List

      class Info < Common::Info
        supported_schemes ['http']
        default_standard_uri SEARCH_URI
      private

        def parse_args(arguments)
          super
        end

        def run(options, uri)
          not_implemented
        end
      end # Info

      class Query < Common::Query
        supported_schemes ['http']
        default_standard_uri SEARCH_URI
      private

        def parse_args(arguments)
          super
        end

        def run(options, uri)
          ebeye = SOAP::EBI::Search::Driver.new(uri)
          ebeye.wiredump_dev = STDERR if options.debug
                    
          begin
            parameters = SOAP::EBI::Search::GetNumberOfResults.new(options.database, options.query)
            result = ebeye.getNumberOfResults(parameters)
          rescue Exception => e
            operation_error(e.message)
          end
          
          hitlimit = 50
          hitindex = options.offset
          hitsleft = [ options.maxcount, result.numberOfResults - hitindex ].min
          
          puts hitindex > 0 ? "#{hitindex+1}-#{hitindex+hitsleft}/#{result.numberOfResults}" : "#{hitsleft}/#{result.numberOfResults}"
          STDOUT.flush
          
          while true
            begin
              parameters = SOAP::EBI::Search::GetResults.new(options.database, options.query, ["id", "description"], hitindex, [ hitsleft, hitlimit ].min)
              result = ebeye.getResults(parameters)
            rescue Exception => e
              operation_error(e.message)
            end

            break if result.arrayOfEntryValues.nil?

            result.arrayOfEntryValues.each do |hit|
              puts hit.join("|")
            end
            STDOUT.flush

            break if result.arrayOfEntryValues.length < [ hitsleft, hitlimit ].min

            hitindex += result.arrayOfEntryValues.length
            hitsleft -= result.arrayOfEntryValues.length
            hitlimit = 100

            break if hitsleft <= 0
          end

        end
      end # Query

      class BLAST < Common::BLAST
        supported_schemes ['http']
        default_standard_uri WUBLAST_URI
      private

        def parse_args(arguments)
          super
        end

        def run(options, uri)
          wublast = SOAP::EBI::WUBLAST::Driver.new(uri)
          wublast.wiredump_dev = STDERR if options.debug
          
          parameters = SOAP::EBI::WUBLAST::Parameters.new(
            options.program, # program
            options.database, # database
            nil, # matrix
            1.0, # exp
            nil, # echofilter
            nil, # filter
            nil, # numal
            nil, # scores
            nil, # sensitivity
            nil, # sort
            nil, # status
            nil, # strand
            "xml", # outformat
            nil, # topcombin
            false, # async
            "erik@lagercrantz.ath.cx" # email
          )
          
          data = ""
          input = SOAP::EBI::WUBLAST::DataArray.new()
          $stdin.each do |line|
            if line[0..0] == '>'
              input << SOAP::EBI::WUBLAST::Data.new("sequence", data) if data.length > 0
              data = ""
            else
              data += line.gsub(/\s/,'')
            end
          end
          input << SOAP::EBI::WUBLAST::Data.new("sequence", data)

          begin
            jobId = wublast.runWUBlast(parameters, input)
            result = wublast.poll(jobId, "toolxml")
          rescue Exception => e
            operation_error(e.message)
          else
            # Base64.decode64(result
            Parsers::StreamParser.new(result, Reporter::EWB.new(spartan)).parse
          end
        end
      end # BLAST

      class Fetch < Common::Fetch
        supported_schemes ['http']
        default_standard_uri FETCH_URI
      private

        def parse_args(arguments)
          super
        end

        def run(options, uri)
          ebi = SOAP::EBI::Fetch::Driver.new(uri)
          ebi.wiredump_dev = STDERR if options.debug

          begin
            response = ebi.fetchBatch(options.database, options.identifiers.join(" "), "default", "raw")
          rescue Exception => e
            operation_error(e.message)
          else
            response.each do |entry|
              puts entry
            end
          end
        end
      end # Fetch

    end # EBI
  end # Command

  module Service
    class EBI < Base      
      def self.default
        self.new("ebi", "EBI Web Services")
      end

      def initialize(identifier, name)
        super(identifier, name)
        self.register_command(Command::EBI::Help.default)
        self.register_command(Command::EBI::List.default)
        self.register_command(Command::EBI::Info.default)
        self.register_command(Command::EBI::Query.default)
        self.register_command(Command::EBI::BLAST.default)
        self.register_command(Command::EBI::Fetch.default)
      end

    end # EBI

    register(EBI.default)
  end # Service

end # DatabaseClient
