#!/usr/bin/env ruby
#
# mrs.rb
# BioX Database Client
#
# Created by Erik Lagercrantz on 2007-08-10.
# Copyright 2007 Erik Lagercrantz.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#

require 'provider/common'

require 'ws/mrs/regular'
require 'ws/mrs/blast'

require 'ostruct'
require 'pathname'

module DatabaseClient

  module Command
    module MRS
      REGULAR_URI = URI.parse('http://mrs.cmbi.ru.nl/mrsws')
      BLAST_URI = URI.parse('http://mrs.cmbi.ru.nl/mrsws-blast')
      
      class Help < Common::Help
        private

        def parse_args(arguments)
          super
        end

        def run(options, uri)
          puts "MRS help"
        end
      end # Help

      class List < Common::List
        supported_schemes ['http', 'file']
        default_standard_uri REGULAR_URI
      private

        def configure_parser
          super
        end
      
        def parse_args(arguments)
          super(arguments)
        end

        def run(options, uri)
          case uri.scheme
          when "file"
            run_local(options, Pathname.new(URI.decode(uri.path)))
          when "http"
            run_web(options, uri)
          else
          end
        end

        def run_local(options, path)
          valid_count = 0
          regex = /\.cmp$/
          path.each_entry do |entry|
            if entry.to_s =~ regex
              database = entry.to_s.gsub(regex, '')
              name = ""
              info = `mrs.pl info -d #{path+database}`
              if ($?.to_i == 0)
                info.scan(/^ *[Nn]ame: *(.*)$/) {|n| name = n; break}
                show_database({
                  'id' => database,
                  'name' => name,
                  'fetch' => true,
                  'metadata' => true,
                  'homology' => false},
                  options)
                valid_count += 1
              end
            end
          end
          if valid_count == 0
            self.error "No valid databanks found."
          end
        end

        def run_web(options, uri)
          mrs = SOAP::MRS::Regular::Driver.new(uri)
          mrs.wiredump_dev = STDERR if options.debug

          parameters = SOAP::MRS::Regular::DatabankInfoParameters.new("all")
          begin
            response = mrs.getDatabankInfo(parameters)
          rescue Exception => e
            operation_error(e.message)
          end

          response.each do |db|
            show_database({
              'id' => db.id,
              'name' => db.name,
              'fetch' => true,
              'metadata' => true,
              'homology' => false},
              options)
          end
        end
      end # List

      class Info < Common::Info
        supported_schemes ['http', 'file']
        default_standard_uri REGULAR_URI
      private

        def configure_parser
          super
        end
    
        def parse_args(arguments)
          super(arguments)
        end

        def run(options, uri)
          case uri.scheme
          when "file"
            run_local(options, Pathname.new(URI.decode(uri.path)))
          when "http"
            run_web(options, uri)
          else
          end
        end
        
        def run_local(options, path)
          command = "mrs.pl info -d '#{path+options.database}'"
          IO.popen command do |pipe|
            %w{HUP INT QUIT TERM}.each {|s| trap(s){Process.kill(s,pipe.pid); exit 2}}
            
            section = nil
            pipe.each do |line|
              case line
              when /^([A-Z][a-z]+( [A-Z][a-z]+)*):\s+(.+)$/
                key, value = $1, $3
                key.downcase!
                key.gsub!(/\s+/, '_')
                spartan.property key, value
              when /^([A-Z][a-z]+( [A-Z][a-z]+)*)[^:]*:$/
                section = $1.downcase.gsub(/\s+/, '_')
              when /^\s*$/
                section = nil
              when /^\s+([a-z]+( [a-z]+)*):\s+(\S.*)$/
                key, value = $1, $3
                key.gsub!(/\s+/, '_')
                if section == 'header'
                  spartan.property key, value
                end
              end
            end
          end
        end
        
        def run_web(options, uri)
          mrs = SOAP::MRS::Regular::Driver.new(uri)
          mrs.wiredump_dev = STDERR if options.debug

          parameters = SOAP::MRS::Regular::DatabankInfoParameters.new(options.database)
          begin
            response = mrs.getDatabankInfo(parameters)
          rescue Exception => e
            operation_error(e.message)
          end
          
          response.each do |database|
            spartan.property 'id', database.id
            spartan.property 'name', database.name
            spartan.property 'url', database.url
            spartan.property 'script', database.script
            spartan.property 'fetch', true
            spartan.property 'metadata', true
            spartan.property 'homology', false
#            spartan.property 'homology', database.blastable

            file = database.files.first
            spartan.property 'uuid', file.uuid
            spartan.property 'version', file.version
            spartan.property 'file', file.path
            spartan.property 'modified', file.modification_date
            spartan.property 'entries', file.entries
            spartan.property 'file_size', file.file_size
            spartan.property 'raw_size', file.raw_data_size
          end
        end # run_web
        
      end # Info

      class Query < Common::Query
        supported_schemes ['http', 'file']
        default_standard_uri REGULAR_URI
      private

        def configure_parser
          super
        end
      
        def parse_args(arguments)
          options = super(arguments)
          options.terms = options.query.split(/\s+/).collect do |term|
            term.gsub(/\w+:/, '')
          end.reject do |term|
            term.match(/\W/)
          end
          options
        end

        def run(options, uri)
          case uri.scheme
          when "file"
            run_local(options, Pathname.new(URI.decode(uri.path)))
          else
            run_web(options, uri)
          end
        end

        def run_local(options, path)
          command = ""
          command += "MRS_DATA_DIR='#{path}' " if path
          command += "mrs_alt_query"
          command += " -d '#{options.database}'" if options.database
          command += " -n '#{options.maxcount}'" if options.maxcount
          command += " -q '#{options.query}'" if options.query
          command += options.terms.collect{|term| " -t '#{term}'"}.join if options.terms
          
          begin
            IO.popen command, "r" do |pipe|
              %w{HUP INT QUIT TERM}.each {|s| trap(s){Process.kill(s,pipe.pid); exit 2}}
              pipe.each do |line|
                case line
                when /^(\d+)\|(\S+)\|(.*)$/
                  spartan.context 'hit' do |h|
                    h.property 'score', $1
                    h.property 'identifier', $2
                    h.property 'description', $3
                  end
                when /^(\d+)\/(\d+)$/
                  spartan.context 'hitcount' do |hc|
                    hc.property 'included', $1
                    hc.property 'total', $2
                  end
                end
              end
            end
          rescue Exception => e
            operation_error(e.message)
          end
        end

        def run_web(options, uri)
          mrs = SOAP::MRS::Regular::Driver.new(uri)
          mrs.wiredump_dev = STDERR if options.debug

          hitlimit = 150
          hitindex = options.offset
          hitsleft = options.maxcount
          firstResult = true

          while true
            begin
              parameters = SOAP::MRS::Regular::FindParameters.new(options.database, options.terms, nil, false, options.query, hitindex, [ hitsleft, hitlimit ].min )
              result = mrs.find(parameters)
            rescue Exception => e
              operation_error(e.message)
            end

            if firstResult
              hitsleft = [ hitsleft, result.count - options.offset ].min
              
              spartan.context 'hitcount' do |hc|
                hc.property 'included', (result.hits.nil? ? 0 : hitsleft)
                hc.property 'total', result.count
              end

              firstResult = false
            end

            break if result.hits.nil?

            result.hits.each do |hit|
              spartan.context 'hit' do |h|
                h.property 'score', hit.score.to_i
                h.property 'identifier', hit.id
                h.property 'description', hit.title
              end
            end

            break if result.hits.length < [ hitsleft, hitlimit ].min

            hitindex += result.hits.length
            hitsleft -= result.hits.length
            hitlimit *= 2

            break if hitsleft <= 0
          end
        end
        
      end # Query

      class BLAST < Common::BLAST
        supported_schemes ['http', 'file']
        default_standard_uri BLAST_URI
      private

        def parse_args(arguments)
          super
        end

        def run(options, uri)
          puts "MRS BLAST"
        end
      end # BLAST

      class Fetch < Common::Fetch
        supported_schemes ['http', 'file']
        default_standard_uri REGULAR_URI
      private

        def configure_parser
          super
        end
      
        def parse_args(arguments)
          super(arguments)
        end

        def run(options, uri)
          case uri.scheme
          when "file"
            run_local(options, Pathname.new(URI.decode(uri.path)))
          when "http"
            run_web(options, uri)
          else
          end
        end
        
        def run_local(options, path)
          options.identifiers.each do |entry|
            exec "mrs.pl entry -d '#{path+options.database}' -e '#{entry}'"
          end
        end

        def run_web(options, uri)
          mrs = SOAP::MRS::Regular::Driver.new(uri)
          mrs.wiredump_dev = STDERR if options.debug

          options.identifiers.each do |entry|
            parameters = SOAP::MRS::Regular::GetEntryParameters.new(options.database, entry, SOAP::MRS::Regular::Format::Plain)
            begin
              result = mrs.getEntry(parameters)
            rescue Exception => e
              operation_error(e.message)
            end

            puts result.entry
          end
        end
      end # Fetch

    end # MRS
  end # Command

  module Service
    class MRS < Base
      def self.default
        self.new("mrs", "Maarten's Retrieval System")
      end

      def initialize(identifier, name)
        super(identifier, name)
        self.register_command(Command::MRS::Help.default)
        self.register_command(Command::MRS::List.default)
        self.register_command(Command::MRS::Info.default)
        self.register_command(Command::MRS::Query.default)
        self.register_command(Command::MRS::BLAST.default)
        self.register_command(Command::MRS::Fetch.default)
      end
    end # MRS

    register(MRS.default)
  end # Service

end # DatabaseClient
