#!/usr/bin/env ruby
#
# ncbiblast.rb
# BioX Database Client
#
# Created by Erik Lagercrantz on 2008-02-21.
# Copyright 2008 Erik Lagercrantz.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#

require 'provider/common'
require 'reporter/ncb'

require 'open-uri'

module DatabaseClient

  module Command
    module NCBIBLAST
      INFO_URI = URI.parse('http://www.ncbi.nlm.nih.gov/blast/Blast.cgi')
      DISPATCH_URI = URI.parse('http://www.ncbi.nlm.nih.gov/Service/dispd.cgi')
      EFETCH_URI = URI.parse('http://eutils.ncbi.nlm.nih.gov/entrez/eutils/efetch.fcgi')
      
      DATABASES =
      [
        {
          'id' => 'nt',
          'name' => 'Nucleotide Collection',
          'moltype' => 'nuc'
        },
        {
          'id' => 'nr',
          'name' => 'Non-Redundant Protein Collection',
          'moltype' => 'prot'
        },
        {
          'id' => 'refseq_rna',
          'name' => 'Reference Sequences (mRNA)',
          'moltype' => 'nuc'
        },
        {
          'id' => 'refseq_genomic',
          'name' => 'Reference Sequences (Genomic)',
          'moltype' => 'nuc'
        },
        {
          'id' => 'refseq_protein',
          'name' => 'Reference Sequences (Protein)',
          'moltype' => 'prot'
        },
        {
          'id' => 'est',
          'name' => 'Expressed Sequence Tags (All)',
          'moltype' => 'nuc'
        },
        {
          'id' => 'est_human',
          'name' => 'Expressed Sequence Tags (Human)',
          'moltype' => 'nuc'
        },
        {
          'id' => 'est_mouse',
          'name' => 'Expressed Sequence Tags (Mouse)',
          'moltype' => 'nuc'
        },
        {
          'id' => 'est_others',
          'name' => 'Expressed Sequence Tags (Other)',
          'moltype' => 'nuc'
        },
        {
          'id' => 'patnt',
          'name' => 'Patent Sequences (Nucleotide)',
          'moltype' => 'nuc'
        },
        {
          'id' => 'pataa',
          'name' => 'Patent Sequences (Protein)',
          'moltype' => 'prot'
        },
        {
          'id' => 'pdb',
          'name' => 'Protein Databank',
          'moltype' => 'prot'
        },
        {
          'id' => 'swissprot',
          'name' => 'Swissprot',
          'moltype' => 'prot'
        },
        {
          'id' => 'gss',
          'name' => 'Genomic Survey Sequences',
          'moltype' => 'nuc'
        },
        {
          'id' => 'htgs',
          'name' => 'High Throughput Genomic Sequences',
          'moltype' => 'nuc'
        },
        {
          'id' => 'sts',
          'name' => 'Sequence Tagged Sites',
          'moltype' => 'nuc'
        },
        {
          'id' => 'wgs',
          'name' => 'Whole-Genome Shotgun Reads',
          'moltype' => 'nuc'
        },
        {
          'id' => 'alu_repeats',
          'name' => 'Human ALU Repeat Elements',
          'moltype' => 'nuc'
        },
        {
          'id' => 'env_nt',
          'name' => 'Environmental Samples (Nucleotide)',
          'moltype' => 'nuc'
        },
        {
          'id' => 'env_nr',
          'name' => 'Environmental Samples (Protein)',
          'moltype' => 'prot'
        }
      ]
      
      class Help < Common::Help
      private
        def run(options, uri)
          puts "Network BLAST help"
        end
      end # Help

      class List < Common::List
        supported_schemes ['file', 'http']
        default_standard_uri INFO_URI
      private

        def parse_args(arguments)
          super
        end

        def run(options, uri)
          # uri.query = "CMD=Info&TARGET=databases"
          # begin
          #   open(uri).each do |line|
          #     if line =~ /^([A-Za-z][A-Za-z._-]*)\s+([0-9]+)\s+(TRUE|FALSE)$/
          #       show_database({
          #         'id' => $1,
          #         'name' => nameify($1),
          #         'fetch' => false,
          #         'metadata' => false,
          #         'homology' => true,
          #         'moltype' => ($3 == 'TRUE' ? 'prot' : 'nuc'),
          #         'format' => 'genbank'},
          #         options)
          #     end
          #   end
          # rescue Exception => e
          #   operation_error(e.message)
          # end
          DATABASES.each do |db|
            db['fetch'] = false
            db['metadata'] = false
            db['homology'] = true
            db['format'] = 'genbank'
            show_database(db, options)
          end
          
          db = {'id' => 'sequences', 'name' => 'Sequences', 'fetch' => true, 'metadata' => false, 'homology' => false, 'format' => 'genbank'}
          show_database(db, options)
        end
      end # List

      class Info < Common::Info
        supported_schemes ['file', 'http']
        default_standard_uri INFO_URI
      private

        def parse_args(arguments)
          super
        end

        def run(options, uri)
#           uri.query = "CMD=Info&TARGET=databases"
#           begin
#             open(uri).each do |line|
#               if line =~ /^(#{options.database})\s+([0-9]+)\s+(TRUE|FALSE)$/
#                 show_database({
#                   'id' => $1,
#                   'name' => nameify($1),
#                   'fetch' => false,
#                   'metadata' => false,
#                   'homology' => true,
#                   'moltype' => ($3 == 'TRUE' ? 'prot' : 'nuc'),
#                   'format' => 'genbank'},
#                   options)
#               end
#             end
#           rescue Exception => e
#             operation_error(e.message)
#           end
          if options.database == 'sequences'
            db = {'id' => 'sequences', 'name' => 'Sequences', 'fetch' => true, 'metadata' => false, 'homology' => false, 'format' => 'genbank'}
          else
            db = DATABASES.find { |d| d['id'] == options.database }
            db['fetch'] = false
            db['metadata'] = false
            db['homology'] = true
            db['format'] = 'genbank'
          end
          show_database(db, options)
        end
      end # Info

      class BLAST < Common::BLAST
        supported_schemes ['file', 'http']
        default_standard_uri DISPATCH_URI
      private

        def parse_args(arguments)
          super
        end
        
        def run(options, uri)
          case uri.scheme
          when "file"
            run_local(options, Pathname.new(URI.decode(uri.path)))
          else
            run_net(options, uri)
          end
        end

        def run_local(options, path)
          ncpu = `sysctl -n hw.ncpu`.to_i
        	outtype = 7 # XML Blast output
        	
        	command = ""
        	command += "cd '#{path}' && " if path
          command += "'" + (options.cmdpath ? options.cmdpath : 'blastall') + "' "
          command += "-p '#{options.program}' " if options.program
          command += "-d '#{options.database}' " if options.database
        	command += "-m '#{outtype}' " if outtype
          command += "-a '#{ncpu}' " if ncpu and ncpu > 0
          
          perform command
        end
        
        def run_net(options, uri)
          outtype = 7 # XML Blast output
        	
          command = ""
          if options.uri
            # command += "SRV_ENGINE_URL='#{options.uri}' "
            command += "CONN_HOST='#{options.uri.host}' "
            command += "CONN_PORT='#{options.uri.port}' "
            command += "CONN_PATH='#{options.uri.path}' "
          end
          command += "'" + (options.cmdpath ? options.cmdpath : 'blastCL3') + "' "
          command += "-p '#{options.program}' " if options.program
          command += "-d '#{options.database}' " if options.database
          command += "-m '#{outtype}' " if outtype
          
          perform command
        end
        
        def perform(command)
          begin
            IO.popen command, "r" do |pipe|
              %w{HUP INT QUIT TERM}.each {|s| trap(s){Process.kill(s,pipe.pid); exit 2}}
              Parsers::StreamParser.new(pipe, Reporter::NCB.new(spartan)).parse
            end
          rescue Exception => e
            operation_error(e.message)
          end
        end
        
      end # BLAST
      
      class Fetch < Common::Fetch
        supported_schemes ['http']
        default_standard_uri EFETCH_URI
      private
        
        def parse_args(arguments)
          super
        end
        
        def run(options, uri)
          options.identifiers.each do |identifier|
            query = {
              'db' => options.database,
              'id' => identifier,
              'retmode' => 'text',
              'rettype' => 'gb'
            }
            uri.query = query.map{|key,val| key+'='+val}.join('&')
            open(uri) do |response|
              response.each do |line|
                puts line
              end
            end
          end
        end
      end # Fetch
      
    end # NCBIBLAST
  end # Command

  module Service
    class NCBIBLAST < Base      
      def self.default
        self.new("ncb", "NCBI BLAST (local install)")
      end

      def initialize(identifier, name)
        super(identifier, name)
        self.register_command(Command::NCBIBLAST::Help.default)
        self.register_command(Command::NCBIBLAST::List.default)
        self.register_command(Command::NCBIBLAST::Info.default)
        self.register_command(Command::NCBIBLAST::BLAST.default)
        self.register_command(Command::NCBIBLAST::Fetch.default)
      end

    end # NCBIBLAST

    register(NCBIBLAST.default)
  end # Service

end # DatabaseClient
