# MRS plugin for creating an EMBL db
#
# $Id: embl.pm 131 2006-08-10 12:02:09Z hekkel $
#
# Copyright (c) 2005
#      CMBI, Radboud University Nijmegen. All rights reserved.
#
# This code is derived from software contributed by Maarten L. Hekkelman
# and Hekkelman Programmatuur b.v. Modified by Erik Lagercrantz.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#        This product includes software developed by the Radboud University
#        Nijmegen and its contributors.
# 4. Neither the name of Radboud University nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE RADBOUD UNIVERSITY AND CONTRIBUTORS
# ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE UNIVERSITY OR CONTRIBUTORS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#

package MRS::Script::embl;

use strict;

our @ISA = "MRS::Script";

sub new
{
	my $invocant = shift;

	my $self = {
		name		=> "eBioX Custom Database",
		url			=> "http://www.ebioinformatics.org/ebiox",
		meta		=> [ 'title' ],
		section		=> "nucleotide",
		@_
	};
	
	return bless $self, "MRS::Script::embl";
}

sub parse
{
	my $self = shift;
	local *IN = shift;
	
	my ($id, $doc, $m, $title);
	
	$m = $self->{mrs};
	
	my $lookahead = <IN>;
	while (defined $lookahead)
	{
		my $line = $lookahead;
		$lookahead = <IN>;

		$doc .= $line;

		chomp($line);

		if ($line eq '//')
		{
			$m->StoreMetaData('title', $title);
			
			$m->Store($doc);
			$m->FlushDocument;

			$id = undef;
			$doc = undef;
			$title = undef;
		}
		elsif ($line =~ /^([A-Z]{2}) {3}(.+)/o)
		{
			my $fld = $1;
			my $text = $2;

			if ($fld eq 'ID')
			{
				my @flds = split(m/;\s*/, $text);

				if (@flds > 5)
				{
					# New EMBL format
					$m->IndexValue('id', $flds[0]);
					$flds[1] =~ m/SV (\d+)/ && $m->IndexNumber('sv', $1);
					$m->IndexWord('topology', lc($flds[2]));
					$m->IndexWord('mt', lc($flds[3]));
					$m->IndexWord('dc', lc($flds[4]));
					$m->IndexWord('td', lc($flds[5]));
					$flds[6] =~ m/(\d+)/ && $m->IndexNumber('length', $1);
				}
				else {
					# Old EMBL format
					my @subflds = split(m/\s+/, $flds[0]);
					$m->IndexValue('id', $subflds[0]);
					$subflds[1] =~ s/standard/STD/g;
					$m->IndexWord('dc', lc($subflds[1]));
					if ($flds[1] =~ m/circular/i) { $m->IndexWord('topology', 'circular'); }
					else { $m->IndexWord('topology', 'linear'); }
					$flds[1] =~ s/circular\s+//i;
					$m->IndexWord('mt', lc($flds[1]));
					$m->IndexWord('td', lc($flds[2]));
					$flds[3] =~ m/(\d+)/ && $m->IndexNumber('length', $1);
				}
			}
			elsif ($fld eq 'SV')
			{
				# Old EMBL format
				$text =~ m/\w+\.(\d+)/ && $m->IndexNumber('sv', $1);
			}
			elsif ($fld eq 'DT')
			{
				$m->IndexTextAndNumbers('dt', $text);
			}
			elsif ($fld eq 'DE')
			{
				$m->IndexText('de', $text);

				$title .= ' ' if defined $title;
				$title .= $text;
			}
			elsif (substr($fld, 0, 1) eq 'R')
			{
				$m->IndexText('ref', substr($line, 5));
			}
			elsif ($fld eq 'FT')
			{
				# avoid indexing the sequences in the translation feature
				
				if ($text =~ m|/([^=]+)="(.+)"|)
				{
					$m->IndexText('ft', $2) if ($1 ne 'translation');
				}
				elsif ($line =~ m|/([^=]+)="(.+)|)
				{
					my $skip = $1 eq 'translation';
					
					$m->IndexText('ft', $2) unless $skip;
					
					while ($lookahead =~ /^FT\s+(.+)/ and substr($1, 0, 1) ne '/')
					{
						$m->IndexText('ft', $1) unless $skip;

						$doc .= $lookahead;
						
						$lookahead = <IN>;
					}
				}
			}
			elsif ($fld ne 'SQ' and $fld ne 'XX')
			{
				$m->IndexText(lc($fld), $text);
			}
		}
	}
}

1;
